@@ -1,3 +1,7 @@
+try:
+    from collections import OrderedDict
+except ImportError:
+    OrderedDict = dict
 from collections import namedtuple
 import re
 
@@ -30,26 +34,32 @@ class Column(object):
     """
     Store metadata about a database column.
     """
-    primary_key_types = (IntegerField, PrimaryKeyField)
+    primary_key_types = (IntegerField, AutoField)
 
     def __init__(self, name, field_class, raw_column_type, nullable,
-                 primary_key=False, db_column=None, index=False, unique=False):
+                 primary_key=False, column_name=None, index=False,
+                 unique=False):
         self.name = name
         self.field_class = field_class
         self.raw_column_type = raw_column_type
         self.nullable = nullable
         self.primary_key = primary_key
-        self.db_column = db_column
+        self.column_name = column_name
         self.index = index
         self.unique = unique
 
+        # Foreign key metadata.
+        self.rel_model = None
+        self.related_name = None
+        self.to_field = None
+
     def __repr__(self):
         attrs = [
             'field_class',
             'raw_column_type',
             'nullable',
             'primary_key',
-            'db_column']
+            'column_name']
         keyword_args = ', '.join(
             '%s=%s' % (attr, getattr(self, attr))
             for attr in attrs)
@@ -61,16 +71,18 @@ class Column(object):
         # Set up default attributes.
         if self.nullable:
             params['null'] = True
-        if self.field_class is ForeignKeyField or self.name != self.db_column:
-            params['db_column'] = "'%s'" % self.db_column
-        if self.primary_key and not self.field_class is PrimaryKeyField:
+        if self.field_class is ForeignKeyField or self.name != self.column_name:
+            params['column_name'] = "'%s'" % self.column_name
+        if self.primary_key and self.field_class is not AutoField:
             params['primary_key'] = True
 
         # Handle ForeignKeyField-specific attributes.
         if self.is_foreign_key():
-            params['rel_model'] = self.rel_model
+            params['model'] = self.rel_model
             if self.to_field:
-                params['to_field'] = "'%s'" % self.to_field
+                params['field'] = "'%s'" % self.to_field
+            if self.related_name:
+                params['backref'] = "'%s'" % self.related_name
 
         # Handle indexes on column.
         if not self.is_primary_key():
@@ -82,7 +94,7 @@ class Column(object):
         return params
 
     def is_primary_key(self):
-        return self.field_class is PrimaryKeyField or self.primary_key
+        return self.field_class is AutoField or self.primary_key
 
     def is_foreign_key(self):
         return self.field_class is ForeignKeyField
@@ -91,7 +103,8 @@ class Column(object):
         return (self.field_class is ForeignKeyField and
                 self.rel_model == "'self'")
 
-    def set_foreign_key(self, foreign_key, model_names, dest=None):
+    def set_foreign_key(self, foreign_key, model_names, dest=None,
+                        related_name=None):
         self.foreign_key = foreign_key
         self.field_class = ForeignKeyField
         if foreign_key.dest_table == foreign_key.table:
@@ -99,6 +112,7 @@ class Column(object):
         else:
             self.rel_model = model_names[foreign_key.dest_table]
         self.to_field = dest and dest.name or None
+        self.related_name = related_name or None
 
     def get_field(self):
         # Generate the field definition for this column.
@@ -118,15 +132,17 @@ class Column(object):
 
 class Metadata(object):
     column_map = {}
+    extension_import = ''
 
     def __init__(self, database):
         self.database = database
+        self.requires_extension = False
 
     def execute(self, sql, *params):
         return self.database.execute_sql(sql, params)
 
     def get_columns(self, table, schema=None):
-        metadata = dict(
+        metadata = OrderedDict(
             (metadata.name, metadata)
             for metadata in self.database.get_columns(table, schema))
 
@@ -138,9 +154,9 @@ class Metadata(object):
         if len(pk_names) == 1:
             pk = pk_names[0]
             if column_types[pk] is IntegerField:
-                column_types[pk] = PrimaryKeyField
+                column_types[pk] = AutoField
 
-        columns = {}
+        columns = OrderedDict()
         for name, column_data in metadata.items():
             columns[name] = Column(
                 name,
@@ -148,7 +164,7 @@ class Metadata(object):
                 raw_column_type=column_data.data_type,
                 nullable=column_data.null,
                 primary_key=column_data.primary_key,
-                db_column=name)
+                column_name=name)
 
         return columns
 
@@ -185,6 +201,7 @@ class PostgresqlMetadata(Metadata):
         1700: DecimalField,
         2950: TextField, # UUID
     }
+    extension_import = 'from playhouse.postgres_ext import *'
 
     def __init__(self, database):
         super(PostgresqlMetadata, self).__init__(database)
@@ -204,6 +221,10 @@ class PostgresqlMetadata(Metadata):
 
     def get_column_types(self, table, schema):
         column_types = {}
+        extension_types = set((
+            postgres_ext.JSONField,
+            postgres_ext.TSVectorField,
+            postgres_ext.HStoreField)) if postgres_ext is not None else set()
 
         # Look up the actual column type for each column.
         identifier = '"%s"."%s"' % (schema, table)
@@ -214,6 +235,8 @@ class PostgresqlMetadata(Metadata):
             column_types[column_description.name] = self.column_map.get(
                 column_description.type_code,
                 UnknownField)
+            if column_types[column_description.name] in extension_types:
+                self.requires_extension = True
 
         return column_types
 
@@ -332,7 +355,7 @@ class SqliteMetadata(Metadata):
         return column_types
 
 
-DatabaseMetadata = namedtuple('DatabaseMetadata', (
+_DatabaseMetadata = namedtuple('_DatabaseMetadata', (
     'columns',
     'primary_keys',
     'foreign_keys',
@@ -340,8 +363,27 @@ DatabaseMetadata = namedtuple('DatabaseMetadata', (
     'indexes'))
 
 
+class DatabaseMetadata(_DatabaseMetadata):
+    def multi_column_indexes(self, table):
+        accum = []
+        for index in self.indexes[table]:
+            if len(index.columns) > 1:
+                field_names = [self.columns[table][column].name
+                               for column in index.columns
+                               if column in self.columns[table]]
+                accum.append((field_names, index.unique))
+        return accum
+
+    def column_indexes(self, table):
+        accum = {}
+        for index in self.indexes[table]:
+            if len(index.columns) == 1:
+                accum[index.columns[0]] = index.unique
+        return accum
+
+
 class Introspector(object):
-    pk_classes = [PrimaryKeyField, IntegerField]
+    pk_classes = [AutoField, IntegerField]
 
     def __init__(self, metadata, schema=None):
         self.metadata = metadata
@@ -367,21 +409,36 @@ class Introspector(object):
         return self.metadata.database.database
 
     def get_database_kwargs(self):
-        return self.metadata.database.connect_kwargs
+        return self.metadata.database.connect_params
+
+    def get_additional_imports(self):
+        if self.metadata.requires_extension:
+            return '\n' + self.metadata.extension_import
+        return ''
 
     def make_model_name(self, table):
         model = re.sub('[^\w]+', '', table)
-        return ''.join(sub.title() for sub in model.split('_'))
+        model_name = ''.join(sub.title() for sub in model.split('_'))
+        if not model_name[0].isalpha():
+            model_name = 'T' + model_name
+        return model_name
 
     def make_column_name(self, column):
-        column = re.sub('_id$', '', column.lower()) or column.lower()
+        column = re.sub('_id$', '', column.lower().strip()) or column.lower()
+        column = re.sub('[^\w]+', '_', column)
         if column in RESERVED_WORDS:
             column += '_'
+        if len(column) and column[0].isdigit():
+            column = '_' + column
         return column
 
-    def introspect(self, table_names=None):
+    def introspect(self, table_names=None, literal_column_names=False):
         # Retrieve all the tables in the database.
-        tables = self.metadata.database.get_tables()
+        if self.schema:
+            tables = self.metadata.database.get_tables(schema=self.schema)
+        else:
+            tables = self.metadata.database.get_tables()
+
         if table_names is not None:
             tables = [table for table in tables if table in table_names]
 
@@ -408,13 +465,27 @@ class Introspector(object):
                 foreign_keys[table] = self.metadata.get_foreign_keys(
                     table, self.schema)
             except ValueError as exc:
-                err(exc.message)
+                err(*exc.args)
                 foreign_keys[table] = []
 
             model_names[table] = self.make_model_name(table)
 
-            for column_name, column in table_columns.items():
-                column.name = self.make_column_name(column_name)
+            lower_col_names = set(column_name.lower()
+                                  for column_name in table_columns)
+
+            for col_name, column in table_columns.items():
+                if literal_column_names:
+                    new_name = re.sub('[^\w]+', '_', col_name)
+                else:
+                    new_name = self.make_column_name(col_name)
+
+                # If we have two columns, "parent" and "parent_id", ensure
+                # that when we don't introduce naming conflicts.
+                lower_name = col_name.lower()
+                if lower_name.endswith('_id') and new_name in lower_col_names:
+                    new_name = col_name.lower()
+
+                column.name = new_name
 
             for index in table_indexes:
                 if len(index.columns) == 1:
@@ -428,6 +499,27 @@ class Introspector(object):
             columns[table] = table_columns
             indexes[table] = table_indexes
 
+        # Gather all instances where we might have a `related_name` conflict,
+        # either due to multiple FKs on a table pointing to the same table,
+        # or a related_name that would conflict with an existing field.
+        related_names = {}
+        sort_fn = lambda foreign_key: foreign_key.column
+        for table in tables:
+            models_referenced = set()
+            for foreign_key in sorted(foreign_keys[table], key=sort_fn):
+                try:
+                    column = columns[table][foreign_key.column]
+                except KeyError:
+                    continue
+
+                dest_table = foreign_key.dest_table
+                if dest_table in models_referenced:
+                    related_names[column] = '%s_%s_set' % (
+                        dest_table,
+                        column.name)
+                else:
+                    models_referenced.add(dest_table)
+
         # On the second pass convert all foreign keys.
         for table in tables:
             for foreign_key in foreign_keys[table]:
@@ -439,9 +531,10 @@ class Introspector(object):
                     dest = None
 
                 src.set_foreign_key(
-                    foreign_key,
-                    model_names,
-                    dest)
+                    foreign_key=foreign_key,
+                    model_names=model_names,
+                    dest=dest,
+                    related_name=related_names.get(src))
 
         return DatabaseMetadata(
             columns,
@@ -450,8 +543,10 @@ class Introspector(object):
             model_names,
             indexes)
 
-    def generate_models(self, skip_invalid=False, table_names=None):
-        database = self.introspect(table_names=table_names)
+    def generate_models(self, skip_invalid=False, table_names=None,
+                        literal_column_names=False, bare_fields=False):
+        database = self.introspect(table_names=table_names,
+                                   literal_column_names=literal_column_names)
         models = {}
 
         class BaseModel(Model):
@@ -467,56 +562,57 @@ class Introspector(object):
 
             primary_keys = []
             columns = database.columns[table]
-            for db_column, column in columns.items():
+            for column_name, column in columns.items():
                 if column.primary_key:
                     primary_keys.append(column.name)
 
-            multi_column_indexes = []
-            column_indexes = {}
-            indexes = database.indexes[table]
-            for index in indexes:
-                if len(index.columns) > 1:
-                    field_names = [columns[column].name
-                                   for column in index.columns]
-                    multi_column_indexes.append((field_names, index.unique))
-                elif len(index.columns) == 1:
-                    column_indexes[index.columns[0]] = index.unique
+            multi_column_indexes = database.multi_column_indexes(table)
+            column_indexes = database.column_indexes(table)
 
             class Meta:
                 indexes = multi_column_indexes
+                table_name = table
 
             # Fix models with multi-column primary keys.
+            composite_key = False
             if len(primary_keys) == 0:
                 primary_keys = columns.keys()
             if len(primary_keys) > 1:
-                Meta.primary_key = CompositeKey([
+                Meta.primary_key = CompositeKey(*[
                     field.name for col, field in columns.items()
                     if col in primary_keys])
+                composite_key = True
 
             attrs = {'Meta': Meta}
-            for db_column, column in columns.items():
+            for column_name, column in columns.items():
                 FieldClass = column.field_class
-                if FieldClass is UnknownField:
+                if FieldClass is not ForeignKeyField and bare_fields:
+                    FieldClass = BareField
+                elif FieldClass is UnknownField:
                     FieldClass = BareField
 
                 params = {
-                    'db_column': db_column,
+                    'column_name': column_name,
                     'null': column.nullable}
-                if column.primary_key and not FieldClass is PrimaryKeyField:
+                if column.primary_key and composite_key:
+                    if FieldClass is AutoField:
+                        FieldClass = IntegerField
+                    params['primary_key'] = False
+                elif column.primary_key and FieldClass is not AutoField:
                     params['primary_key'] = True
                 if column.is_foreign_key():
                     if column.is_self_referential_fk():
-                        params['rel_model'] = 'self'
+                        params['model'] = 'self'
                     else:
                         dest_table = column.foreign_key.dest_table
-                        params['rel_model'] = models[dest_table]
+                        params['model'] = models[dest_table]
                     if column.to_field:
-                        params['to_field'] = column.to_field
+                        params['field'] = column.to_field
 
                     # Generate a unique related name.
-                    params['related_name'] = '%s_%s_rel' % (table, db_column)
-                if db_column in column_indexes and not column.is_primary_key():
-                    if column_indexes[db_column]:
+                    params['backref'] = '%s_%s_rel' % (table, column_name)
+                if column_name in column_indexes and not column.is_primary_key():
+                    if column_indexes[column_name]:
                         params['unique'] = True
                     elif not column.is_foreign_key():
                         params['index'] = True
